package service

import (
	"context"
	"crypto/md5"
	"flag"
	"os"
	"testing"

	"go-common/app/service/main/filter/conf"
	"go-common/app/service/main/filter/model"

	. "github.com/smartystreets/goconvey/convey"
)

var (
	ctx     = context.Background()
	service *Service
)

func TestMain(m *testing.M) {
	flag.Set("conf", "../cmd/filter-service-test.toml")
	var err error
	if err = conf.Init(); err != nil {
		panic(err)
	}
	service = New()
	defer service.Close()
	os.Exit(m.Run())
}

func TestService(t *testing.T) {
	Convey("Filter", t, func() {
		var (
			msg          = "Twitter脑残\xbd\xbd\xbd\xbd脑残麻烦到洗洗脑子再来，cnm,典型的脑残患者"
			typeID       int64
			id, oid, mid int64 = 1, 1, 1
			keys               = []string{}

			err   error
			level int8
			fmsg  string
			limit int
			// typeIDs []int64
			hitRules []string
		)

		t.Logf("%s, %d, %d", msg, len([]byte(msg)), len([]rune(msg)))
		fmsg, level, _, _, _, _, err = service.Filter(ctx, "reply", msg, typeID, id, oid, mid, keys, 1)
		So(err, ShouldBeNil)
		So(fmsg, ShouldNotBeEmpty)
		So(level, ShouldBeGreaterThan, 0)
		So(hitRules, ShouldNotBeEmpty)
		So(limit, ShouldEqual, 0)
	})

	Convey("Filter hit", t, func() {
		var (
			msg  = article
			tpID int64

			err      error
			hitRules []string
		)
		hitRules, err = service.Hit(ctx, "article", msg, tpID)
		So(err, ShouldBeNil)
		So(hitRules, ShouldNotBeEmpty)
	})

	Convey("Filter hit V3", t, func() {
		var (
			msg  = article
			tpID int64

			err  error
			hits []*model.HitRes
		)
		hits, err = service.HitV3(ctx, "article", msg, tpID, 0)
		So(err, ShouldBeNil)
		So(hits, ShouldNotBeEmpty)
	})

	Convey("Filter area mpost", t, func() {
		var (
			msg  = article
			tpID int64

			err error
			// fmsg     string
			level int8
		)
		_, level, _, err = service.FilterArea(ctx, "article", msg, tpID)
		So(err, ShouldBeNil)
		So(level, ShouldBeGreaterThan, 0)
	})
}

func BenchmarkFilterShort(b *testing.B) {
	var (
		msg  = "毛泽东sfdsffds习近平毛泽东sfdsffds习近平毛泽东s"
		err  error
		keys = []string{}
	)
	for i := 0; i < b.N; i++ {
		if _, _, _, _, _, _, err = service.Filter(ctx, "reply", msg, 0, 0, 0, 0, keys, 1); err != nil {
			b.Fatal(err)
		}
	}
}

func BenchmarkFilterLong(b *testing.B) {
	var (
		msg  = "2011 年，还在哈佛念 MBA 的陈郢刚刚结束了在宜信的暑期实习，和唐宁交流自己的创业想法。唐宁问他：「你是打算软着陆呢，还是硬着陆？」陈郢疑惑了一下：「软着陆和硬着陆是什么意思？」唐宁说：「你看，宜信现在发展得不错，我们一起再干几年，你就不差钱了。到时候再出去创业，这就叫软着陆。或者你现在就出去自己闯，这是硬着陆。我提醒你，创业可不是一件容易事。」陈郢说：「那我还是硬着陆吧。」从农村到城市2012 年，陈郢还在帮唐宁探索「宜农贷」等农村普惠金融业务。陈郢对农村有一种特殊感情，在哈佛商学院求学期间，他便深入各国农村做过研究。出来创业，他的第一个想法也是农村电商——要把好东西卖给老乡们。从徐小平、唐宁等大佬处拿到投资后，陈郢便一个猛子扎到了农村，开始了长达 3 年的奋斗。陈郢可能是中国最早做农村电商的创业者之一，他们在农村建立各种社群，像在城市运作「社群电商」一样把货品推给农民。环境熟悉了，理念探究了，社群建立了，渠道也打通了，然而陈郢却越来越怀疑起一个本质问题：老乡们到底需不需要农村电商？「经过几年的实践，东西虽然能卖出去，我们做的事情却并不为人称道。对于大多数农村居民来说，有没有电商带来的产品，他们的生活不会发生太大变化。」陈郢总结道。陈郢以为，虽然近年来农村的经济增长有目共睹，但中国多数农村的消费水平和城市还存在较大差距。结合在宜信的经验，陈郢发觉，与其给农民带去消费品，让农民「花钱」，不如给农民带去资金、方法、渠道，让他们「赚钱」——不管通过普惠金融还是其他方式。于是，2016 年春天，陈郢和整个团队开始谋求转型。一开始的思路不很确定，彼时，一批秭归脐橙正在当地售卖，陈郢和哈佛商学院同学，他的合伙人常青试吃时，感觉这橙子口感相当好，可以卖给城里人。这时，一位跟随陈郢多年的创业伙伴站了出来，对他说，秭归橙卖到城里，绝对不丢人。这位伙伴叫王呈祥，也就是日后大名鼎鼎的寻味师「橙先生」。橙先生是秭归本地人，家中贩橙为业，对秭归橙品质颇为了解。橙先生说，赣南脐橙出名早，又有地方政府大力保护，以致成了品牌；而秭归橙其实并不比赣南橙差。一个不算秘密的秘密是，外来客商到秭归收购脐橙，贴上赣南橙标签去卖，当地政府和果农为眼前小利竟默不作声。于是大众市场只知赣南，不知秭归。橙先生说，如果能大大方方地把秭归脐橙直接卖到城里，这种误会也许能够消除。寻味师“橙先生”说干就干。常青是 Bon Juice 创始人，也是一位「宝妈」，更是一位社群领袖。她在北京的几个社群里推销了一下秭归橙，居然只用 2 小时就卖出了县城本地一周的销量。陈郢和常青面面相觑：真的要转型了。看来，还是把好东西卖给城里人，让农民赚钱更靠谱儿。寻味师：每月两个 SKU由于在农村时便以社群形式做电商，陈郢「杀回」城市时，同样火速采取了社群战略。彼时是 2016 年春，「消费升级」的概念还不为人所熟知，但陈郢在农村的教训告诉他：要迎合当地消费者的需求，不能一厢情愿地灌输。优选、精致的商品对农民而言或许太早，但对城里人，尤其北京这种大城市的居民来说，可谓势在必行了。然而精致的商品从哪儿来，谁能承担优中选优的重大责任？橙先生的成功给了陈郢一个念头。经历半年左右的试探，有好东西最终确立了「寻味师」这一产品开发岗位。产品团队试吃会，盲测海苔寻味师是什么？说白了就是产品经理，然而他们的职责不是开发软件，而是开发水果、海鲜等。听起来挺玄乎的，实际上他们都是些什么人呢？在招聘网站上写个「寻味师」，莫不是让应聘者一脸问号？陈郢回答了我的疑问：确实不太好招。以现有的寻味师团队来看，他们大多数是在某个商品领域摸爬滚打多年的采购「老江湖」，事实上最初的几位寻味师年龄确实偏大；然而，这并不意味着经验丰富的经销商或采购员就完全可以胜任「寻味师」了。如果说传统采购负责的是「渠道」，那么寻味师负责的却是「产品」。一般采购的工作模式，是找到可靠的供应商，然后在供应商提供的商品中，把有需要的一股脑揽过来。寻味师却绝不能「以不变应万变」，不能找到一家上游便无尽地挖掘其能力，而是要为每个具体的产品找到「最好的上游」。只是找到最好的供应商还不够。寻味师要结合产品的特点与用户的需要，「从头到尾」向用户负责一个完整的 SKU。从确立品种、产地开始，怎样采摘，怎样包装，怎样入库，物流如何，定价多少……以及怎么向消费者展示、宣传，怎么教消费者「正确的吃法」，寻味师都要将它们��������������������一体。陈郢提醒我，有好东西这种「从头到尾」的「负责」到了怎样的程度呢？大型的生鲜电商，为了保持供应的稳定、品质的标准，比较依赖大型供应商或当地批发市场。这种模式较为传统，能够满足多数用户的需要，但不意味着能够满足所有人。这就给了一些更小、更灵活的新兴电商以挑战机会。那么，近年兴起的特色农产品电商，以及一些淘宝店和微商，采用的是何种突破方式？为了向消费者强调产品的新鲜，它们往往采用了更激进的「产地直发」模式，比如秭归橙就从秭归采摘，快递直发消费者家里。虽然这种模式不适合规模巨大的传统电商，但无疑将鲜度的「上限」提高了一个档次。多数人认为，近年在新型生鲜电商中流行的「产地直发」，已经是速度最快的方式，不存在优化空间。但是在有好东西看来，这种「流行做法」仍然是不够的。「不可控的环节太多了。即便是顺丰这种值得信赖的公司，从秭归到北京，出库入库加起来也要有个十来次，这中间转手太多、变数太多，对高度追求质量的产品而言是不可接受的，」陈郢说，「在我们看来，这就不叫『负责』了。」因此，有好东西花了不少心思建立自己的仓储、物流体系，把货物送到北京自己的仓库里，再从北京统一配送给本地消费者；将来在江浙沪开卖，也会在江浙沪建仓。只把最后一步交给宅配公司，出库后全都实现当日达、次日达，这样，「产品只有不到 12 小时的时间在第三方配送承运商的手上，」陈郢说，「即便如此，我们也有专人监控包裹的实时投递情况。」只有这样，才能尽量保证「卖出的全是精品」。而陈郢还在思索的是，怎么把这「最后一步」的「不到 12 小时」也纳入「产品」的范畴。一般的采购，即便是「老江湖」也很难如陈郢所愿，把产品的各个环节敲得恰到好处。因此，陈郢的做法是「试试看」。「寻味师」入职以后，一个月内和有经验的寻味师共同负责一个 SKU，下个月独自负责一个 SKU。直到成绩受到认可，每个寻味师每月开发的新品，平均也就在 2 种左右。每月开发 2 个新品！你不敢相信传统采购能这么任性。即便如此，有好东西成立至今发掘的 300 来个 SKU 中，也只保留了 100 个左右。他们将质量视作生命，也正因如此，100 个优中选优的 SKU 才能撑起 6000 的平均日单，和 2000 万的月流水。社群、生活体验与甄选师在有好东西的理念中，好产品绝不等于好货源。举例来说，鸡蛋是一种配送难度较高的商品，很少有人会「一个一个」地网购鸡蛋。而出售一盒 30 个的鸡蛋，难度与「一个一个」卖却不可同日而语。因为，即便在配送过程中只有 1% 的碎蛋率，30 个一盒的鸡蛋却有高达 26% 的可能产生碎蛋，26% 的可能意味着 26% 的投诉，而一款好评率只有 74% 的产品绝对算不上「好产品」。然而，通过挑选品种、设计包装、控制物流，有好东西仍能成功地卖出盒装鸡蛋爆款。你很难想象这是一个 70 人左右的小团队能完成的事情。分门别类的独到包装这时候，陈郢就会说，他们靠的不只是这 70 个全职员工。在偌大的北京，有 10000 个「兼职」或说「志愿者」在为有好东西服务，他们就是不同于寻味师的「甄选师」。你可以将甄选师理解为网红，理解为节点用户；你可以将他们理解为微商中的佼佼者，也可以说他们就是一群名不见经传的自媒体。无论如何，甄选师在各自的社群里充当着社群领袖，就像「秭归脐橙」一炮打响时功不可没的常青。在陈郢的定位中，甄选师有能力也有意愿向他们各自的「VIP」会员们传达商品价值，并最终产生商业价值。这些「VIP」或者与甄选师有亲密关系（家人、同事），或者是网红甄选师的粉丝；但是，一旦甄选师向他们推荐商品，他们之间的关系就不再是表达与聆听，而是服务与被服务，单纯聆听的粉丝也就跃升成了接受服务的「会员」。而会员与甄选师之间的信任关系，会经由商品的确认，逐渐转化为消费者与有好东西之间的信任关系。这便弥合了高速、高效、覆盖广、规模大的现代电商里最大的一道裂痕：信任。具体到有好东西，他们的用户定位主要是城市中的家庭用户，尤其是年轻妈妈。用户如此，甄选师自然也如此。大多甄选师都是宝妈，他们以对母婴产品的高要求挑剔地选购有好东西提供的产品，再将这种「优中选优再选优」的结果传达给他们的粉丝。是的，甄选师从来不负责把有好东西的所有产品卖给所有人。甄选师有自己的态度，有自己的偏好，他们只对粉丝输出自己喜欢的单品。他们中的大部分本来只是有好东西的普通会员，在意识到有好东西是值得信赖的电商后，才能放心从事「甄选师」这一职务。在有好东西的内部交流 app 中，东奔西走一个月的寻味师带着产品归来，给甄选师开「发布会」——这一步骤本来在微信群中即可进行，然而甄选师人数将近万人的现在，微信群便行不通了。甄选师了解到产品是什么，了解到哪里好、怎么吃、怎么玩以后，如果他们喜欢，就会下单，然后推荐给自己所属社群中的其他成员。由于整个售卖过程中有着传统电商无法企及的大量互动以及情感关系和信赖关系，有好东西的产品不仅有着极高的好评率，也有着极高的晒单率。宝妈的创造能力是惊人的，「花式晒单」往往伴随着「花式晒娃」甚至「花式晒猫」，而一切的晒单都成为了再传播的素材——你甚至不用去教甄选师怎么卖货。强信任突破了品类的限制。起初是水果，后来变成各类生鲜，再后来连包装食品也算上；如今，非食品类的产品也出现在有好东西上，甚至「抽象」的产品也开始出现，比如亲子活动。高质量不捆绑于某个产品，而捆绑于一种理想，一种生活体验。在非社群的电商中，「用户买食物觉得好，就会去买生活用品；买生活用品觉得好，就甚至会去买保险」的逻辑是行不通的，因为用户不会对电商产生无限制的信任，一般只会信任某一垂直品类。但是，当信任连结于一个具体的人，你就有更高的几率去尝试他推荐的一切——甄选师就是这样一种角色。结语「那么，在哪里才能买得到呢？」有好东西服务的主要是北京城市居民，其中最大量的便是宝妈。会员通过社群认识并加入有好东西，他们在微信看货，在微信下单。有好东西也有自己的 app，但这个 app 目前并非流量入口，只用作甄选师的内部交流。当然，甄选师本身就是会员的一部分。正因如此，你甚至很难说有好东西是卖货的，他们从事的更像是针对绝对数量不大的会员的服务。你甚至很难说有好东西是个电商，因为他们简直像媒体——大量、高频、深度的提醒和互动，换成一般电商你烦不烦？陈郢的投资人阵营包括真格徐小平、大众点评张涛和宜信唐宁等人。为了更加完善物流体系，此刻，他们正在谋求下一轮融资。我向陈郢提出疑问：有好东西现有的模式，应该不会只局限于北京。陈郢说，有好东西正在考虑探索华东地区，毕竟「江浙沪市场比北京还大」。此外，随着商品目录的不断更新，有好东西在将来可能不只是「水果只在这儿买就好了」的生鲜优选电商，而可能是「优质的家庭生活体验，『有好东西』就够了」。"
		err  error
		keys = []string{}
	)
	for i := 0; i < b.N; i++ {
		if _, _, _, _, _, _, err = service.Filter(ctx, "reply", msg, 0, 0, 0, 0, keys, 1); err != nil {
			b.Fatal(err)
		}
	}
}

func BenchmarkKeyFilter(b *testing.B) {
	for i := 0; i < b.N; i++ {
		service.Filter(ctx, "tag", "cxw2", 0, 0, 0, 0, []string{"cid:3"}, 1)
	}
}

func BenchmarkArticle(b *testing.B) {
	var err error
	for i := 0; i < b.N; i++ {
		if _, err = service.Hit(ctx, "article", article, 0); err != nil {
			b.Fatal(err)
		}
	}
}

func BenchmarkAreaMPOST(b *testing.B) {
	var err error
	for i := 0; i < b.N; i++ {
		if _, _, _, err = service.FilterArea(ctx, "article", article, 0); err != nil {
			b.Fatal(err)
		}
	}
}

func TestHash(t *testing.T) {
	Convey("hash", t, func() {
		hash := hashContent(article)
		t.Log(hash)

		bytes := md5.Sum([]byte(article))
		t.Log(bytes)
	})
}

var (
	article = `原标题：潮起鹭岛 风正帆悬——写在金砖国家领导人厦门会晤闭幕之际

	大厦之门，金砖闪亮。五色风帆，激扬世界。

	9月3日至5日，金砖国家领导人厦门会晤成功举行。

	金砖五国领导人会晤，10个新兴市场国家和发展中国家展开对话，全球约1000名工商界人士聚首，习近平主席多次发表讲话……共识在沟通中凝聚，方向在擘画中清晰。

	这是指引金砖合作崭新航程的重要会晤。

	这是构建更加广泛伙伴关系的成功实践。

	这是推动国际秩序公正合理的有力行动。

	金砖巨轮从厦门再出发，乘风破浪，扬帆远航，必将迎来发展繁荣的光明未来。



	这是9月5日拍摄的厦门国际会议中心。 新华社记者张国俊摄

	迈上新起点，标注新航向，开创金砖合作第二个“金色十年”

	厦门国际会议中心，面朝大海，形若巨轮，见证金砖合作又一历史性时刻。

	“让我们共同努力，推动金砖合作从厦门再次扬帆远航，开启第二个‘金色十年’的大门，使金砖合作造福我们五国人民，惠及各国人民！”

	4日上午，厦门国际会议中心凤凰厅，金砖五国领导人围坐在圆形会议桌旁，习近平主席的主旨讲话回顾合作历程，勾画未来蓝图。



	9月4日，金砖国家领导人第九次会晤在厦门国际会议中心举行。国家主席习近平主持会晤并发表题为《深化金砖伙伴关系开辟更加光明未来》的重要讲话。新华社记者谢环驰摄

	——这是金砖发展史上承前启后、继往开来的里程碑式会晤。

	十年树木，木已挺拔。

	10年来，金砖合作从无到有，由浅入深，五国经济总量增长179%，贸易总额增长94%，城镇化人口增长28%，成为促进世界经济增长、推动全球秩序变革、维护国际和平稳定的关键力量。

	金砖国家用一份亮丽的成绩单，有力回击了“褪色论”“空心论”等无端质疑。

	“厦门会晤标志着金砖合作步入第二个十年，金砖国家的发展将对世界经济继续带来积极影响。”英国《金融时报》刊文称。

	“金砖合作之所以得到快速发展，关键在于找准了合作之道。”习近平主席回顾金砖发展之路，感慨满怀。



	9月4日，金砖国家领导人第九次会晤在厦门国际会议中心举行。新华社记者马占成摄

	——互尊互助，携手走适合本国国情的发展道路；

	——秉持开放包容、合作共赢的精神，持之以恒推进经济、政治、人文合作；

	——倡导国际公平正义，同其他新兴市场国家和发展中国家和衷共济，共同营造良好外部环境。

	习近平主席的精辟阐述，得到与会嘉宾的高度认同，在世界范围引发强烈回响。

	面向第二个十年，金砖合作将继续夯基垒台、立柱架梁。



	9月3日，2017年金砖国家工商论坛开幕式在厦门举行。 新华社记者谢环驰摄

	“致力于推进经济务实合作”“致力于加强发展战略对接”“致力于推动国际秩序朝更加公正合理方向发展”“致力于促进人文民间交流”……习近平主席提出“四个致力于”，为金砖合作的未来，指明航向。

	巴西总统特梅尔说，习近平主席的讲话全面分析世界大势，为金砖国家未来发展作出系统规划。

	“习近平主席的讲话给企业界巨大信心，金砖机制一定会为企业间合作拓展更大空间。”参加金砖国家工商论坛的岚桥集团董事长叶成表示，企业今后将继续参与海外基础设施建设，通过港口、能源等项目，推动与金砖国家企业进行更广泛的合作。

	——这是金砖国家凝聚新共识、开创新局面的重要宣誓。

	十年淬炼，成色更足。

	如何在新形势下更好地推进相互合作，是金砖国家面临的重大战略任务。

	据统计，2016年金砖国家对外投资1970亿美元，但只有5.7%发生在五国之间，表明金砖合作潜力还没有充分释放出来。

	面向下一个十年，经济务实合作前景广阔。

	加强贸易投资合作，加强财金合作，促进金融机构和金融服务网络化布局，推动科技创新合作，推动发展绿色和低碳经济……会晤发表的《金砖国家领导人厦门宣言》，涉及70余项共识，合作设想着眼机制化、实心化，含金量进一步提升。

	会晤期间，习近平主席宣布，中方将设立首期5亿元人民币金砖国家经济技术合作交流计划，用于加强经贸等领域政策交流和务实合作。向新开发银行项目准备基金出资400万美元，支持银行业务运营和长远发展。



	9月4日，金砖国家领导人第九次会晤在厦门国际会议中心举行。这是金砖国家领导人集体合影。新华社记者张铎摄

	“感谢中国的大力支持，让新开发银行的资金规模不断壮大，从而更好地支持金砖国家的合作和发展。”金砖国家新开发银行行长卡马特说。

	面向下一个十年，发展战略对接事关全局。

	当前，金砖各国处在相近发展阶段，具有相同发展目标，都已进入经济爬坡过坎的时期，面临结构性改革、反贫困、推动创新等一系列共性问题。

	汇聚合力，方能生发动力。

	习近平主席指出，我们应该在大局上谋划、关键处落子，本着共商、共建、共享原则，寻找发展政策和优先领域的契合点，继续向贸易投资大市场、货币金融大流通、基础设施大联通目标迈进。

	“通过政策协调，金砖国家能够有效应对逆全球化、贸易壁垒带来的挑战，合力推动解决和平赤字、发展赤字、治理赤字。”中国国际经济交流中心副总经济师徐洪才说。

	——这是金砖伙伴同心同向、蓄势待发的崭新起点。

	4日，金砖国家领导人共同出席金砖国家文化节启动仪式暨文化图片展。习近平主席和其他金砖国家领导人在各国印章柱上按下印章，对面屏幕上同时显示出金砖国家中文国名篆刻印文，共同启动文化节开幕式。



	9月4日，国家主席习近平在厦门出席金砖国家文化节启动仪式暨文化图片展。 新华社记者庞兴雷摄

	国之交在于民相亲。

	新加坡《海峡时报》称，厦门会晤一大焦点是扩大人文交流，增强金砖国家之间的联系。

	“只有深耕厚植，友谊和合作之树才能枝繁叶茂。加强我们五国人文交流，让伙伴关系理念扎根人民心中，是一项值得长期投入的工作。这项工作做好了，将使金砖合作永葆活力。”习近平主席说。

	从举行运动会、电影节、文化节，到举办传统医药高级别会议，今年以来一系列丰富多彩的活动让金砖国家人民心贴得更近、手拉得更紧。

	金砖国家不是碌碌无为的清谈馆，而是知行合一的行动队。

	“爱拼才会赢”。会晤期间，习近平主席引用闽南话，为金砖国家合作更上一层楼加油鼓劲。


	“欢迎更多金砖国家企业来埃及，来非洲，来阿拉伯世界！”“我们要搭上数字化革命的快车！”……

	从埃及总统塞西到几内亚总统孔戴，在金砖国家工商论坛上，两位国家领导人不约而同地当起各自国家的“首席推销员”。



	9月4日，2017年金砖国家工商论坛在厦门举行“互联互通”专题研讨。 这是几内亚总统孔戴出席专题研讨。 新华社记者李鑫摄

	5日上午，包括这两位总统在内的5位新兴市场国家和发展中国家领导人，与金砖国家领导人就合作共赢大计展开一场坦诚而深入的对话。

	金砖国家从来不是封闭的俱乐部。从2010年南非加入，到2013年起金砖国家领导人每年同地区国家领导人举行对话会，金砖国家的历史脉动始终与广大新兴市场国家和发展中国家的命运紧密联系在一起。

	正如习近平主席所指出的：“我们应该发挥自身优势和影响力，促进南南合作和南北对话，汇聚各国集体力量，联手应对风险挑战。我们应该扩大金砖合作的辐射和受益范围，推动‘金砖+’合作模式，打造开放多元的发展伙伴网络，让更多新兴市场国家和发展中国家参与到团结合作、互利共赢的事业中来。”



	9月4日，2017年金砖国家工商论坛在厦门举行“互联互通”专题研讨。这是专题研讨结束后，发言嘉宾准备合影。 新华社记者李鑫摄

	“金砖+”，这是来自中国的新倡议，是惠及世界的新事业。

	“世界的格局正由量变转变为质变，金砖国家代表的新兴市场对世界经济治理的话语权和影响力将会上升，历史的任务落到了金砖国家的身上。”中国现代国际关系研究院研究员陈凤英说。

	一箭易断，十箭难折。

	“共同构建开放型世界经济”“共同落实2030年可持续发展议程”“共同把握世界经济结构调整的历史机遇”“共同建设广泛的发展伙伴关系”，习近平主席阐明了新兴市场国家和发展中国家加强团结协作的中国主张。

	推动“金砖+”，既扩大了金砖国家的“朋友圈”，更拓展了金砖国家的“辐射圈”。

	厦门会晤期间，前不久发出的厦门至莫斯科的中欧班列正行驶在欧亚大陆上，上万公里的路程将21世纪海上丝绸之路和丝绸之路经济带连接起来，辐射沿线发展中国家。



	中欧（厦门—莫斯科）直达货运班列从厦门自贸片区海沧货场开出（4月21日摄）。 新华社记者林善传摄

	一花不是春，孤雁难成行。

	从开通电气化铁路到修建高速公路，得益于金砖合作等因素的助力，非洲迎来新的发展机遇。

	“金砖合作的发展给所有被边缘化的国家带来希望，我们与金砖国家的伙伴关系将给非洲大陆带来新面貌。”孔戴说。

	推动“金砖+”，既有利于发展中国家“抱团取暖”，更可为世界经济增长“雪中送炭”。

	金砖合作并非独善其身，发展中国家合作最终将造福整个世界。

	习近平主席指出，作为具有全球影响力的合作平台，金砖合作的意义已超出五国范畴，承载着新兴市场国家和发展中国家乃至整个国际社会的期望。

	国际货币基金组织发布的报告预计，在中国和印度经济增长带动下，2017年新兴经济体对全球经济增长的贡献将超过四分之三。未来4年，非洲将是全球经济增长第二快的地区。



	9月2日，在金砖国家领导人厦门会晤新闻中心，几名来自非洲的记者在工作。 新华社记者宋为伟摄

	“世界经济的风险在增加，这给所有经济体的发展带来挑战，但也促使各国联合起来共渡难关。”俄罗斯战略研究所经济研究中心副主任卡拉塔耶夫说，如果各国取长补短，利用自己的优势为自己和其他国家谋福利，将推动世界经济的长久发展。

	推动“金砖+”，既稳固了南南合作的大平台，更搭建起南北合作的新桥梁。

	在南南合作援助基金项下提供5亿美元的援助，利用国际发展知识中心、南南合作与发展学院等平台，同各国加强发展经验交流和能力建设合作，在未来一年为其他发展中国家提供4万个来华培训名额……中国宣布一系列举措推进南南合作。

	在重大问题上集体发声，提高发展中国家在国际机构中的权重……通过有效对话和务实行动，金砖国家不仅赢得了南北对话的主动权，还开启了南北合作的新篇章。

	和则通，通则兴，兴则利。厦门会晤期间，习近平主席逐一会见与会各国领导人，深入沟通，坦诚交流，扩大共识，凝聚力量。



	9月5日，国家主席习近平在厦门国际会议中心主持新兴市场国家与发展中国家对话会并发表重要讲话。这是对话会前，习近平同出席对话会的各国领导人集体合影。 新华社记者庞兴雷摄

	北京师范大学金砖国家合作中心主任王磊认为，“金砖+”模式既加强了内部战略伙伴关系，又打造了全球发展伙伴关系，不仅是发展中国家间合作的平台，也将成为全球多极化合作的崭新平台。

	把握新机遇，勾画新图景，迈向人类发展更加灿烂的明天

	中国国家统计局最新发布的数据显示，8月份中国制造业PMI达到51.7%，总体保持稳中向好发展态势。

	与此同时，俄罗斯中央银行上调第三季度经济增长预测至1.7%—2.2%，巴西经济今年连续两个季度正增长……

	“金砖国家对全球经济增长作出了重大贡献，这是他们非常重要的成就。”印度德里大学政治学教授苏迪尔·古普塔说。

	当世界经济深陷国际金融危机泥潭、举步维艰之时，金砖国家是引领全球经济复苏的重要引擎。



	9月4日，在厦门机场上空，一架飞机飞向蓝天。 新华社记者魏培全摄

	如今，尽管世界经济正渐渐走出阴霾，但面对大变革、大调整的时代洪流，和平与冲突、多边与单边、开放与保护仍在激烈角力，未来之路依旧迷雾重重。

	建设一个什么样的世界？如何建设这个世界？在增长贡献之外，国际社会更加渴求中国和其他新兴市场国家提供破解之道。

	开好方先要把好脉，把好脉先要找准病灶。

	“由于近年来世界经济处于疲弱期，发展失衡、治理困境、公平赤字等问题显得更加突出，保护主义和内顾倾向有所上升。世界经济和全球经济治理体系进入调整期，面临新的挑战。”习近平主席的讲话一语中的。

	进入调整期的世界经济，总体增长速度趋缓，市场扩展力减弱，贸易保护主义趋势加剧，金融市场动荡性增加，不稳定、不确定、不平衡的因素增多。

	面临新挑战的世界经济，迫切需要推进结构转型，提高金融机构防范和应对风险能力，深化国际合作，完善全球经济治理。



	厦门夜景（8月24日摄）。 新华社记者姜克红摄

	“在全面参与全球治理的进程中，金砖国家作为一支建设性力量，不是对国际体系推倒重来，而是推动渐进性改革，提升全球治理的能力。”王磊说。

	这是以自身实践为完善全球治理开辟的崭新征程——

	由小到大，由近及远，由金砖国家延伸至更多国家和地区，世界经济体系的发展走向正在发生影响深远的巨变。

	工商论坛上，与会嘉宾谈经验、提建议、亮观点，希望通过推动开放让各国共享发展机遇、汲取发展力量。

	“抵制全球化、推行贸易保护主义是幼稚、自私的行为。金砖国家要继续坚持全球化进程，在经济、科技和民间交往等领域推动彼此融合，实现经济发展，改善人民生活水准。”南非工商理事会主席伊克巴尔·苏尔维说。



	8月26日拍摄的厦门国际会展中心（拼接照片）。 新华社记者姜克红摄

	这是以使命担当为推动国际体系变革踏出的康庄大道——

	“新兴市场国家和发展中国家的发展，不是要动谁的奶酪，而是要努力把世界经济的蛋糕做大。”习近平主席的论断鲜明有力。

	唯有包容才能让进步持久。

	“金砖国家与各国建立起共赢关系，这将对构建和谐世界十分有利。世界期待金砖国家在国际事务决策中发挥作用，为全球化的未来指明方向。”澳大利亚思环顾问公司首席执行官戴维·托马斯说。

	海纳百川，有容乃大。

	从让新兴市场国家和广大发展中国家搭上自身发展的快车、便车，到G20机制中坚定站在发展中国家阵营，中国始终致力于变革全球治理体系中不公正、不合理的安排，增加新兴市场国家和发展中国家的发言权。

	“中国与金砖国家在国际场合共同发声，将发展议题带入全球治理核心领域，深刻地影响着国际秩序变革。”中国人民大学重阳金融研究院执行院长王文说。



	从鼓浪屿远望厦门岛（8月31日摄）。 新华社记者张国俊摄

	斗转星移，潮起潮落。

	厦门，短短30多年，从沿海小城蜕变为高素质、高颜值的活力之城。厦门的成功故事，正是中国改革开放历程的一个生动缩影。

	以厦门会晤为新的起点，展望第二个“金色十年”，金砖国家务实合作一定会创造一个个成功故事，更好造福五国人民，为世界发展和人类进步贡献更多正能量。`
)
